import sys
from qt import *

class TimeProgressBar(QProgressBar):
    def __init__(self, totalSteps, parent = None, name = None):
        QProgressBar.__init__(self, totalSteps, parent, name)
        self.time = QTime()
        self.lastTime = 0
        
    def setProgress(self, progress):
        if progress == 0:
            # starting
            self.time.start()
            self.lastTime = -1
        QProgressBar.setProgress(self, progress)

    def setIndicator(self, indicator, progress, totalSteps):
        #print "setIndicator", indicator, type(indicator), progress, totalSteps
        elapsed = self.time.elapsed()
        if self.lastTime == elapsed:
            return False
        self.lastTime = elapsed
        if progress * 20 < totalSteps and elapsed < 5000:
            # Less than 5 percent complete and less than 5 secs have elapsed.
            indicator = "Measuring..."
            #indicator = QString("Measuring...") # wont work either
        else:
            rest = int(float(totalSteps)/progress * elapsed) - elapsed
            time = QTime(0, 0, 0)
            time = time.addSecs(rest/1000)
            indicator = time.toString()
        #print "->", indicator, type(indicator)
        return True

        
class TimeProgressDialog(QProgressDialog):
    def __init__(self, labelText, cancelButtonText, totalSteps, 
                 creator = None, name = None, modal = False, f = 0):
        QProgressDialog.__init__(self, labelText, cancelButtonText, totalSteps,
                                 creator, name, modal, f)
        bar = TimeProgressBar(totalSteps, self)
        self.setBar(bar)

        
class Test(QWidget):
    def __init__(self):
        QWidget.__init__(self)
        self.timer = QTimer()
        self.connect(self.timer, SIGNAL("timeout()"), self.progress)
        self.dialog = TimeProgressDialog("Doing something", "Cancel", 100, self)
        self.dialog.setProgress(0)
        self.connect(self.dialog, SIGNAL("canceled()"), qApp, SLOT("quit()"))
        self.count = 0
        self.timer.start(1000)

    def progress(self):
        print "progress", self.count
        if self.count == 100:
            qApp.quit()
        else:
            self.count += 1
            self.dialog.setProgress(self.count)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    w = Test()
    QObject.connect(qApp, SIGNAL("lastWindowClosed()"), qApp, SLOT("quit()"))
    app.exec_loop()
